__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from bottle import request, response
from chembl_beaker.beaker.core_apps.svgImages.impl import _ctab2svg, _smiles2svg
import base64

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2svg/<ctab>', method=['OPTIONS', 'GET'], name="ctab2svg")
@app.route('/ctab2svg/<ctab>/<size>', method=['OPTIONS', 'GET'], name="ctab2svg")
@app.route('/ctab2svg/<ctab>/<size>/<legend>', method=['OPTIONS', 'GET'], name="ctab2svg")
def ctab2svg(ctab, size=200, legend=''):
    """
Converts CTAB to SVG vector graphic. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles. Size is the optional size of image in pixels (default value is 200 px).
Legend is optional label in the bottom of image.
    """

    size = int(size)
    data = base64.urlsafe_b64decode(ctab)
    response.content_type = 'image/svg+xml'
    return _ctab2svg(data,size,legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2svg', method=['OPTIONS', 'POST'], name="ctab2svg")
def ctab2svg():
    """
Converts CTAB to SVG vector graphic. CTAB is either single molfile or SDF file. Size is the optional size of
image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(request.forms.get('size', 200))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    legend=request.params.get('legend','')
    response.content_type = 'image/svg+xml'
    return _ctab2svg(data,size,legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2svg/<smiles>', method=['OPTIONS', 'GET'], name="smiles2svg")
@app.route('/smiles2svg/<smiles>/<size>', method=['OPTIONS', 'GET'], name="smiles2svg")
@app.route('/smiles2svg/<smiles>/<size>/<legend>', method=['OPTIONS', 'GET'], name="smiles2svg")
def smiles2svg(smiles, size=200, legend=''):
    """
Converts SMILES to SVG vector graphic. This method accepts urlsafe_base64 encoded string containing single or
multiple SMILES optionally containing header line, specific to *.smi format. Size is the optional size of image in
pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(size)
    data = base64.urlsafe_b64decode(smiles)
    if not data.startswith('SMILES Name'):
        data = "SMILES Name\n" + data
    response.content_type = 'image/svg+xml'
    return _smiles2svg(data,size,legend)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2svg', method=['OPTIONS', 'POST'], name="smiles2svg")
def smiles2svg():
    """
Converts SMILES to SVG vector graphic. This method accepts single or multiple SMILES or *.smi file. Size is the
optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    data = request.body.read()
    if not data.startswith('SMILES Name'):
        data = "SMILES Name\n" + data
    size = int(request.forms.get('size', 200))
    legend=request.params.get('legend','')
    response.content_type = 'image/svg+xml'
    return _smiles2svg(data,size,legend)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from bottle import request, response
import base64
from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.rasterImages.impl import _ctab2image, _smiles2image

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2image/<ctab>', method=['OPTIONS', 'GET'], name="ctab2image")
@app.route('/ctab2image/<ctab>/<size>', method=['OPTIONS', 'GET'], name="ctab2image")
@app.route('/ctab2image/<ctab>/<size>/<legend>', method=['OPTIONS', 'GET'], name="ctab2image")
def ctab2image(ctab, size=200, legend=''):
    """
Converts CTAB to PNG image. CTAB is urlsafe_base64 encoded string containing
single molfile or concatenation of multiple molfiles. Size is the optional size of image in pixels (default value
is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(size)
    data = base64.urlsafe_b64decode(ctab)
    response.content_type = 'image/png'
    ret = _ctab2image(data,size,legend)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/ctab2image', method=['OPTIONS', 'POST'], name="ctab2image")
def ctab2image():
    """
Converts CTAB to PNG image. CTAB is either single molfile or SDF file. Size is the optional size of image in pixels
(default value is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(request.forms.get('size', 200))
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    legend=request.params.get('legend','')
    response.content_type = 'image/png'
    ret = _ctab2image(data,size,legend)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2image/<smiles>', method=['OPTIONS', 'GET'], name="smiles2image")
@app.route('/smiles2image/<smiles>/<size>', method=['OPTIONS', 'GET'], name="smiles2image")
@app.route('/smiles2image/<smiles>/<size>/<legend>', method=['OPTIONS', 'GET'], name="smiles2image")
def smiles2image(smiles, size=200, legend=''):
    """
Converts SMILES to PNG image. This method accepts urlsafe_base64 encoded string containing single or multiple
SMILES optionally containing header line, specific to *.smi format. Size is the optional size of image in pixels
(default value is 200 px). Legend is optional label in the bottom of image.
    """

    size = int(size)
    data = base64.urlsafe_b64decode(smiles)
    if not data.startswith('SMILES Name'):
        data = "SMILES Name\n" + data
    response.content_type = 'image/png'
    ret = _smiles2image(data,size,legend)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/smiles2image', method=['OPTIONS', 'POST'], name="smiles2image")
def smiles2image():
    """
Converts SMILES to PNG image. This method accepts single or multiple SMILES or *.smi file. Size is the optional
size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
    """

    data = request.body.read()
    if not data.startswith('SMILES Name'):
        data = "SMILES Name\n" + data
    size = int(request.forms.get('size', 200))
    legend=request.params.get('legend','')
    response.content_type = 'image/png'
    ret = _smiles2image(data,size,legend)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

#-----------------------------------------------------------------------------------------------------------------------



# $Id$
#
#  Copyright (C) 2013 Michal Nowotka and Greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import re
import json
from rdkit import Chem
from rdkit.Chem.Draw.canvasbase import CanvasBase
from rdkit.Chem.Draw.MolDrawing import MolDrawing, DrawingOptions
from rdkit.Chem.AllChem import Compute2DCoords

class Canvas(CanvasBase):
  """
  The output of this can be inserted in a web page that has raphael loaded as follows:

'''
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <title>Raphael Molecules demo</title>
        <link rel="stylesheet" href="demo.css" type="text/css" media="screen">
        <link rel="stylesheet" href="demo-print.css" type="text/css" media="print">
        <script src="raphael-min.js"></script>
        <style media="screen">
            body {
                margin: 0;
                padding: 0;
                text-align: center;
            }
            h1 {
                font-weight: 400;
                height: 5%%;
            }
            #canvas {
                height: 480px;
                margin: 0 auto;
                text-align: left;
                width: 640px;
            }
            #code {
                font-family: Consolas, Monaco, "Lucida Console", monospace;
                height: 4em;
                margin: 10px;
                padding: 0;
                width: 90%%;
            }
            #run {
                font-size: 2em;
            }
        </style>
        <script>
            window.onload = function () {
                var paper = Raphael("canvas", 600, 600);
                paper.add(%(JSON)s);
            };
        </script>
    </head>
    <body>
        <h1>Raphael Molecule Demo</h1>
        <div id="canvas"></div>
    </body>
</html>
'''

  """
#------------------------------------------------------------------------------

  def __init__(self, size=None):
    self.json=''
    self.objects = []
    self.size=size
    self.scale=1

#------------------------------------------------------------------------------

  def flush(self):
    self.json = json.dumps(self.objects)

#------------------------------------------------------------------------------
    
  def _addCanvasLine(self,p1,p2,color,**kwargs):
    line = dict()
    line['type'] = 'path'
    if kwargs.get('dash',(0,0)) == (0,0):
        line['path'] = "M%s,%sL%s,%s" % (p1[0], p1[1], p2[0], p2[1])
    else:
        dash = kwargs['dash']
        pts = self._getLinePoints(p1,p2,dash)     
        currDash = 0
        dashOn = True
        path = ''
        while currDash<(len(pts)-1):
          if dashOn:
            p1 = pts[currDash]
            p2 = pts[currDash+1]
            path += "M%s,%sL%s,%s" % (p1[0], p1[1], p2[0], p2[1])
          currDash+=1
          dashOn = not dashOn
        line['path'] = path  
        
    line['stroke'] = 'rgb' + str(color)
    line['stroke-width'] = self.scale*kwargs.get('linewidth',1)    
    return line  

#------------------------------------------------------------------------------

  def addCanvasLine(self,p1,p2,color=(0,0,0),color2=None,**kwargs):
    color = tuple(map(lambda x: x*255, color))
    color2 = tuple(map(lambda x: x*255, color2))
    if color2 and color2!=color:
      mp = (p1[0]+p2[0])/2.,(p1[1]+p2[1])/2.
      line1 = self._addCanvasLine(p1,mp,color,**kwargs)
      line2 = self._addCanvasLine(mp,p2,color2,**kwargs)
      self.objects.append(line1)
      self.objects.append(line2)
    else:
      line = self._addCanvasLine(p1,p2,color,**kwargs)
      self.objects.append(line)

#------------------------------------------------------------------------------    

  def addCanvasText(self,text,pos,font,color=(0,0,0),**kwargs):
    color = tuple(map(lambda x: x*255, color))
    bgColor = tuple(map(lambda x: x*255,kwargs.get('bgColor',(1,1,1))))

    orientation=kwargs.get('orientation','E')
    text = re.sub(r'\<.+?\>','',text)
    print text,orientation
    w,h = font.size * len(text),font.size
    bw,bh = w,h*1.4
    xp,yp=pos[0],pos[1]
    if orientation=='E':
      xp += w/2
    elif orientation=='W':
      xp -= w/2
    tex = dict()
    tex['type'] = 'text'
    tex['x'] = xp 
    tex['y'] = yp
    tex['text'] = text
    tex['font-size'] = font.size
    tex['stroke'] = 'rgb' + str(color)
    if font.weight=='bold':
        tex['font-weight'] = "bold"
    backgroundRect = dict()
    backgroundRect['type'] = 'rect'

    if bgColor is not None:
      backgroundRect['x'] = xp - w / 2
      backgroundRect['y'] = yp - h / 2
      backgroundRect['width'] = w
      backgroundRect['height'] = h
      backgroundRect['fill'] = 'rgb' + str(bgColor)
      backgroundRect['stroke'] = 'rgb' + str(bgColor)
      self.objects.append(backgroundRect)    
    self.objects.append(tex)
    return bw,bh,w*pos[2]

#------------------------------------------------------------------------------
    
  def addCanvasPolygon(self,ps,color=(0,0,0),fill=True,stroke=False,**kwargs):
    if not fill and not stroke: return
    color = tuple(map(lambda x: x*255, color))
    polygon = dict()
    polygon['type'] = 'path'
    path = ''
    path += "M%s,%s" % (ps[0][0],ps[0][1])
    for p in ps[1:]:
      path += "L%s,%s" % (p[0],p[1])
    path += 'Z'  
    polygon['path'] = path 
    
    if fill:
        polygon['fill'] = 'rgb' + str(color)
    if stroke:
        polygon['stroke'] = 'rgb' + str(color)     
    self.objects.append(polygon)

#------------------------------------------------------------------------------      

  def addCanvasDashedWedge(self,p1,p2,p3,dash=(2,2),color=(0,0,0),
                           color2=None,**kwargs):
    color = tuple(map(lambda x: x*255, color))                      
    wedge = dict()
    wedge['stroke-width'] = self.scale*kwargs.get('linewidth',1)
    wedge['stroke'] = 'rgb' + str(color)
    wedge['type'] = 'path'
                     
    dash = (3,3)
    pts1 = self._getLinePoints(p1,p2,dash)
    pts2 = self._getLinePoints(p1,p3,dash)

    if len(pts2)<len(pts1): pts2,pts1=pts1,pts2

    path = ''
    for i in range(len(pts1)):
      path += "M%s,%s" % (pts1[i][0],pts1[i][1])
      path += "L%s,%s" % (pts2[i][0],pts2[i][1])

    wedge['path'] = path
    self.objects.append(wedge) 

#------------------------------------------------------------------------------

  def addCircle(self,center,radius,color=(0,0,0),fill=True,stroke=False,alpha=1.0,
                **kwargs):
    if not fill and not stroke: return
    color = tuple(map(lambda x: x*255, color))
    circle = dict()
    circle['type'] = 'circle'
    circle['cx'] = center[0]
    circle['cy'] = center[1]
    circle['r'] = radius
    if fill:
        circle['fill'] = 'rgb' + str(color)
    if stroke:
        circle['stroke'] = 'rgb' + str(color)
    if alpha:
        circle['opacity'] = alpha      
    self.objects.append(circle)
    
#------------------------------------------------------------------------------

def MolToJSON(mol, size=(300,300), kekulize=True, wedgeBonds=True,
               fitImage=False, options=None, **kwargs):
  if not mol:
    raise ValueError,'Null molecule provided'

  canvas = Canvas(size=size)

  if options is None:
    options = DrawingOptions()
  if fitImage:
      options.dotsPerAngstrom = int(min(size) / 10)
  options.wedgeDashedBonds = wedgeBonds
  try:
    drawer = MolDrawing(canvas=canvas,drawingOptions=options)
  except TypeError:
    drawer = MolDrawing(canvas=canvas)

  if kekulize:
    mol = Chem.Mol(mol.ToBinary())
    Chem.Kekulize(mol)

  if not mol.GetNumConformers():
    Compute2DCoords(mol)

  drawer.AddMol(mol,**kwargs)
  try:
    drawer.AddLegend(kwargs.get('legend', ''))
  except AttributeError:
    pass

  canvas.flush()
  return canvas.json

#------------------------------------------------------------------------------


__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from rdkit import Chem
from rdkit.Chem import MCS
from chembl_beaker.beaker.utils.io import _parseMolData

#-----------------------------------------------------------------------------------------------------------------------

def _mcs(data,params):
    ms = _parseMolData(data)
    if not ms:
        return
    if len(ms) == 1:
        if bool(int(params.get('asSmiles','0'))):
            print 'SMARTS'
            return Chem.MolToSmiles(ms[0])
        else:
            print 'SMILES'
            return Chem.MolToSmarts(ms[0])

    atomCompare=params.get('atomCompare','elements')
    bondCompare=params.get('bondCompare','bondtypes')
    ringMatchesRingOnly=bool(int(params.get('ringMatchesRingOnly','0')))
    completeRingsOnly=bool(int(params.get('completeRingsOnly','0')))
    threshold=params.get('threshold',None)
    if threshold:
        threshold=float(threshold)
    try:
        mcs = MCS.FindMCS(ms,
                          atomCompare=atomCompare,
                          bondCompare=bondCompare,
                          ringMatchesRingOnly=ringMatchesRingOnly,
                          completeRingsOnly=completeRingsOnly,
                          threshold=threshold)
    except TypeError:
        mcs = MCS.FindMCS(ms,
                          atomCompare=atomCompare,
                          bondCompare=bondCompare,
                          ringMatchesRingOnly=ringMatchesRingOnly,
                          completeRingsOnly=completeRingsOnly)
    res = mcs.smarts
    if bool(int(params.get('asSmiles','0'))):
        p = Chem.MolFromSmarts(res)
        for m in ms:
            if m.HasSubstructMatch(p):
                match = m.GetSubstructMatch(p)
                res = Chem.MolFragmentToSmiles(m,atomsToUse=match,isomericSmiles=True,canonical=False)
                break
    return res

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

try:
    import cairo
    cffi = False
except ImportError:
    import cairocffi
    cairocffi.install_as_pycairo()
    cffi = True
    import io
    import cairo

import StringIO
from rdkit.Chem.Draw import cairoCanvas
from rdkit.Chem import Draw
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _parseSMILESData
from chembl_beaker.beaker.utils.chemical_transformation import _computeCoords

#-----------------------------------------------------------------------------------------------------------------------

def _mols2svg(mols,size,legend):

    _apply(mols, _computeCoords)

    molsPerRow=min(len(mols),4)
    totalWidth=molsPerRow*size
    totalHeight=molsPerRow*size
    if cffi and cairocffi.version <= (1,10,0) :
        imageData = io.BytesIO()
    else:
        imageData = StringIO.StringIO()
    surf = cairo.SVGSurface(imageData,totalWidth,totalHeight)
    ctx = cairo.Context(surf)
    for i, mol in enumerate(mols):
        tx = size*(i%molsPerRow)
        ty = size*(i//molsPerRow)
        ctx.translate(tx, ty)
        canv = cairoCanvas.Canvas(ctx=ctx, size=(size,size), imageType='svg')
        leg = mol.GetProp("_Name") if mol.HasProp("_Name") else legend
        Draw.MolToImage(mol, size=(size,size), legend=leg, canvas=canv)
        canv.flush()
        ctx.translate(-tx, -ty)
    surf.finish()
    return imageData.getvalue()

#-----------------------------------------------------------------------------------------------------------------------

def _ctab2svg(data,size,legend):
    return _mols2svg(_parseMolData(data), size, legend)

#-----------------------------------------------------------------------------------------------------------------------

def _smiles2svg(data,size,legend):
    return _mols2svg(_parseSMILESData(data), size, legend)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from bottle import request, response
from chembl_beaker.beaker import config

#-----------------------------------------------------------------------------------------------------------------------

class EnableCors(object):
    name = 'enable_cors'
    api = 2

    def apply(self, fn, context):
        def _enable_cors(*args, **kwargs):
            # set CORS headers
            response.headers['Access-Control-Allow-Origin'] = config.get('access_control_allow_origin', '*')
            response.headers['Access-Control-Allow-Methods'] = config.get('access_control_allow_methods',
                                                                                        'GET, POST, PUT, OPTIONS')
            response.headers['Access-Control-Allow-Headers'] = config.get('access_control_allow_headers',
                                                    'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token')

            if request.method != 'OPTIONS':
                # actual request; reply with the actual response
                return fn(*args, **kwargs)

        return _enable_cors

#-----------------------------------------------------------------------------------------------------------------------